{% extends "base.tf.j2" %}

{# All Available Blocks #}

{# Env collection level resource #}
{# {% block application %} #}
{# {% endblock application %} #}

{# Per active environment #}
{# {% block environment_resources %} #}
{# {% endblock environment_resources %} #}

{# Per active service #}
{# {% block service_resources %} #}
{# {% endblock service_resources %} #}

{# Per active|paused environment #}
{# {% block active_and_paused_envs %} #}
{# {% endblock active_and_paused_envs %} #}

{# Custom tf #}
{# {% block custom %} #}
{# {% endblock custom %} #}
